<?
##    -------------------------------------------------------------------------------------
##    $Id: ProxyObject.php,v 1.6 2002/12/22 13:26:20 jonen Exp $
##    -------------------------------------------------------------------------------------
##    $Log: ProxyObject.php,v $
##    Revision 1.6  2002/12/22 13:26:20  jonen
##    + added support of tangram independent id (e.g. Data::UUID) toggled by option at conrtuctor
##
##    Revision 1.5  2002/12/18 22:36:49  jonen
##    + added support to get remote objects via backend via 'guid'
##    + renamed '_loadBackend' to '_loadRemote'
##    + constructor now accepts additional options:
##      + remote: try to get object via '_loadRemote' ...
##      + oid: use the given identifier as an 'oid' when doing '_loadRemote'
##      + guid: use the given identifier as a 'guid' when doing '_loadRemote'
##
##    Revision 1.4  2002/12/12 02:46:31  joko
##    + state (session) gets flushed when data was successfully loaded from remote side
##    + fixed _loadBackend
##    + fixed flushState
##
##    Revision 1.3  2002/12/06 04:12:54  joko
##    + replaced 'xyzCache' through 'xyzProxy'
##    + function store(...)
##
##    Revision 1.2  2002/12/05 21:44:09  joko
##    + debugging
##
##    Revision 1.1  2002/12/01 17:23:58  joko
##    + initial check-in
##
##    Revision 1.11  2002/11/12 06:05:58  cvsjoko
##    + renamed class: Text_Encode -> TextEncode
##
##    Revision 1.10  2002/10/29 19:15:33  cvsjoko
##    - moved utf8/iso-conversion to lib/utils/Text_Encode.php
##
##    Revision 1.9  2002/10/26 12:32:36  cvsjoko
##    - removed debugging via "print"
##    + added generic logging via PEAR::Log
##
##    Revision 1.8  2002/10/22 09:51:38  cvsmax
##    + moved semi-method 'createBackend' to method $site->user->create at User.class.php
##
##    Revision 1.7  2002/10/21 18:27:09  cvsjoko
##    - manually disabled any form of caching
##
##    Revision 1.6  2002/10/17 03:48:47  cvsmax
##    + new
##       + function _createBackend	create new Backend
##       + function _create		api-create
##
##    Revision 1.5  2002/10/16 03:37:54  cvsjoko
##    + bugfix: wrong comparison in restriction to save to array
##
##    Revision 1.4  2002/10/10 03:04:23  cvsjoko
##    + no debug-output
##
##    Revision 1.3  2002/10/10 03:03:27  cvsjoko
##    + bugfix: save object to cache only if payload from backend isn't empty
##
##    Revision 1.2  2002/10/10 02:40:06  cvsjoko
##    + new level of data-caching (session and persistant)
##        + function _loadState	loads data from session
##        + function _saveState	saves data to session
##        + function save		api-save (session & backend)
##        + function _commit	saves data to backend
##    + handy utils
##        + function _setAttributes
##        + function flushState
##
##    Revision 1.1  2002/10/09 00:51:39  cvsjoko
##    + new
##    -------------------------------------------------------------------------------------

class ProxyObject {

  var $objectId;
  // purpose ...
  var $meta;
  var $payload;
  var $attributes;
  var $backend;

  function ProxyObject($objectId = "", $options = array() ) {
    session_register_safe("proxy");
    $this->backend = new Remote;
    $this->_init($objectId, $options);
  }

  function _init($objectId="", $options = array() ) {
    if ($objectId) {
      $this->objectId = $objectId;
      $this->meta = $options;
      
      // set default load mechanismn
      if( $this->meta[remote] && ((!$this->meta[oid] && !$this->meta[guid]) || ($this->meta[oid] && $this->meta[guid])) ) {
        $this->meta[oid] = 1;
      }

      $this->load();
      //$this->_saveProxy();
    }
  }

  function load() {
    if (!$this->_loadState()) {
      if (!$this->_loadProxy()) {

        // just load object from remote side if its flagged to be a remote one ...
        if ($this->meta[remote]) {
          $this->_loadRemote();
        }

      }
    }
  }  

  function save($data, $type) {
    $this->_setAttributes($data);
    $this->_saveState();
    if ($type == 'commit') {
      $this->_commit();
    }
    if ($type == 'create') {
      $this->_create();
    }
  }  

  function flush() {
    $this->flushState();
    $this->flushProxy();
  }      

  function _commit() {
    $this->_saveBackend($this->attributes);
    $this->flushState();
    $this->flushProxy();
  }  


  // TODO: make this work
  /*
  function _create() {
    $this->_createBackend($this->attributes);
    $this->flushState();
    $this->flushProxy();
  }  
  */

  function getAttributes() {
    if (!$this->meta[decoded]) {
      $this->_decode();
      $this->_saveState();
    }
    return $this->attributes;
  }

  function _setAttributes($data) {
    $this->attributes = $data;
  }

  function flushProxy() {
	connectdb();
    $sql = "DELETE FROM f_proxy WHERE oid='$this->objectId'";
    send_sql($sql);
  }
  
  function flushState() {
    global $proxy;
    unset($proxy[$this->objectId]);
    $this->meta[decoded] = 0;
  }

  function _loadState() {
    global $proxy;
    logp(get_class($this) . "->_loadState()", LOG_DEBUG);
    if ($this->attributes = $proxy[$this->objectId]) {
      //print "_loadState:" . dumpVar($this->attributes);
      $this->meta[decoded] = 1;
      // TODO: make a parameter from this (0 deactivates session-layer)
      return 0;
    }
  }

  function _saveState() {
    global $proxy;
    logp(get_class($this) . "->_saveState()", LOG_DEBUG);
    $proxy[$this->objectId] = $this->attributes;
    //print "_saveState: " . dumpVar($this->attributes);
    // TODO: throw exception-message back to user if operation fails
  }

  function _loadProxy() {
    logp(get_class($this) . "->_loadProxy()", LOG_DEBUG);
    connectdb();
    $sql = "SELECT payload FROM f_proxy WHERE oid='$this->objectId'";
    if ($res = send_sql($sql)) {
      $row = mysql_fetch_array($res, MYSQL_ASSOC);
      if ($row) {
        $this->payload = $row[payload];
        // TODO: make a parameter from this (0 deactivates mysqldb-layer)
        return 0;
      }
    }
  }
  
  // TODO: use PEAR here
  function _saveProxy() {
    logp(get_class($this) . "->_saveProxy()", LOG_DEBUG);
	connectdb();
	if ($this->payload) {
      //$sql = "INSERT INTO f_proxy SET payload='$this->payload' WHERE oid='$this->objectId'";
      $sql = "INSERT INTO f_proxy SET oid='$this->objectId', payload='$this->payload'";
      if (!send_sql($sql)) {
        $sql = "UPDATE f_proxy SET payload='$this->payload' WHERE oid='$this->objectId'";
        send_sql($sql);
      }
    }
  }
  
  function _loadRemote() {
    logp(get_class($this) . "->_loadRemote()", LOG_DEBUG);
    
    // TODO: test backend for reachability first (eventually cache this information and "reset" it by another party)
    
    // check for guid or oid
    if($this->meta[guid]) {
      $args = array( guid => $this->objectId, classname => $this->meta[classname] );
      $result = $this->backend->send('getObjectByGuid', $args );
    }
    if($this->meta[oid]) {
      $result = $this->backend->send('getObject', $this->objectId);
    }
    
    if ($result) {
      //print "result: " . dumpVar($result) . "<br>";
      if (count($result) == 0) { return; }
      
      // ----- move this to _encode some times:  $this->_encode()
      $encoder = new TextEncode($result);
      $encoder->toISO();
      if ($_GET[debug]) {
        print dumpVar($result);
      }
      $this->payload = serialize($result);
      // ----- move this to _encode some times
      
      $this->_saveProxy();
      //print "oid: $this->objectId<br>";
      $this->flushState();
    } else {
      print "Error in _loadRemote!!!<br>";
    }
    
  }
  
  function _saveBackend($result) {
    logp(get_class($this) . "->_saveBackend()", LOG_DEBUG);

    //$encoder = new TextEncode($result);
    //$encoder->toUTF8();

    // check for guid or oid
    if($this->meta[guid]) {
      $args = array( 'guid' => $this->objectId, 'classname' => $this->meta[classname], 'data' => $result );
      $response = $this->backend->send('saveObjectByGuid', $args, array( utf8 => 1) );
    }
    if($this->meta[oid]) {
      $response = $this->backend->send('saveObject', array('oid' => $this->objectId, 'data' => $result), array( utf8 => 1)  );
    }
  }

  function _decode() {
    // fill attributes-hashtable from message-hashtable
    if (!$this->payload) { return; }
    //if ($this->attributes = $backend->decodeData($this->payload)) {
    if ($this->attributes = unserialize($this->payload)) {
      $this->meta[decoded] = 1;
    }
  }
  
  function store($struct) {
    $this->payload = serialize($struct);
    $this->_saveProxy();
  }

}

?>